#include <FastLED.h>
#include <SPI.h>
#include <Adafruit_PN532.h>

// === Paramètres ===
#define NOMBRE_LEDS         24
#define BROCHE_DATA         6
#define LONGUEUR_TRAINEE    4
#define LUMIERE_MAX         200
#define LUMIERE_MIN         30
#define DELAI_ESCARGOT      300      // radar lent = temps pour NFC
#define NFC_SS_PIN          10
#define DUREE_EXPLOSION     1500
#define DUREE_AFFICHAGE     5000
#define BROCHE_MOSFET       4
#define DUREE_MOSFET        5000     // Durée d’activation du MOSFET (en ms)

CRGB leds[NOMBRE_LEDS];
Adafruit_PN532 nfc(NFC_SS_PIN);

// === UID badges ===
uint8_t uid_valide[] = {0x04, 0x57, 0xFD, 0x8D, 0x74, 0x00, 0x00};
uint8_t uid_test[]   = {0xB5, 0x4F, 0x44, 0x05};

// === Radar ===
int position_actuelle = 0;
unsigned long dernier_tick = 0;

// === Badge / affichage
bool badgeActif = false;
bool explosionFaite = false;
int typeBadge = 0; // 0 = aucun, 1 = test, 2 = valide, 3 = autre
unsigned long tBadge = 0;

void radarLED() {
  FastLED.clear();
  for (int i = 0; i < LONGUEUR_TRAINEE; i++) {
    int pos = (position_actuelle - i + NOMBRE_LEDS) % NOMBRE_LEDS;
    int intensite = map(i, 0, LONGUEUR_TRAINEE - 1, LUMIERE_MAX, LUMIERE_MIN);
    leds[pos] = CRGB(intensite, intensite, intensite);
  }
  FastLED.show();
  position_actuelle = (position_actuelle + 1) % NOMBRE_LEDS;
}

void allumerCouleur(CRGB couleur) {
  fill_solid(leds, NOMBRE_LEDS, couleur);
  FastLED.show();
}

void explosionCouleurs() {
  for (int i = 0; i < NOMBRE_LEDS; i++) {
    leds[i] = CHSV(random(0, 255), 255, 255);
  }
  FastLED.show();
}

void effetTestRVBTiers() {
  int tier = NOMBRE_LEDS / 3;
  for (int i = 0; i < NOMBRE_LEDS; i++) {
    if (i < tier) leds[i] = CRGB::Red;
    else if (i < 2 * tier) leds[i] = CRGB::Green;
    else leds[i] = CRGB::Blue;
  }
  FastLED.show();
}

bool comparerUID(uint8_t *a, uint8_t *b, uint8_t taille) {
  for (uint8_t i = 0; i < taille; i++) {
    if (a[i] != b[i]) return false;
  }
  return true;
}

void traiterBadge(uint8_t *uid, uint8_t uidLength) {
  if (uidLength == sizeof(uid_valide) && comparerUID(uid, uid_valide, uidLength)) {
    typeBadge = 2;
    explosionFaite = false;
    explosionCouleurs();
    tBadge = millis();
    badgeActif = true;
    digitalWrite(BROCHE_MOSFET, HIGH); // Activer MOSFET
  }
  else if (uidLength == sizeof(uid_test) && comparerUID(uid, uid_test, uidLength)) {
    typeBadge = 1;
    effetTestRVBTiers();
    tBadge = millis();
    badgeActif = true;
  }
  else {
    typeBadge = 3;
    allumerCouleur(CRGB::Red);
    tBadge = millis();
    badgeActif = true;
  }
}

void setup() {
  FastLED.addLeds<NEOPIXEL, BROCHE_DATA>(leds, NOMBRE_LEDS);
  FastLED.clear(); FastLED.show();

  pinMode(BROCHE_MOSFET, OUTPUT);
  digitalWrite(BROCHE_MOSFET, LOW); // Désactivé au démarrage

  nfc.begin();
  nfc.setPassiveActivationRetries(0x01); // lecture rapide
  nfc.SAMConfig();
}

void loop() {
  unsigned long maintenant = millis();

  // === Affichage badge actif
  if (badgeActif) {
    if (typeBadge == 2 && !explosionFaite && maintenant - tBadge > DUREE_EXPLOSION) {
      allumerCouleur(CRGB::Green);
      explosionFaite = true;
      tBadge = maintenant;
    }

    if ((typeBadge != 2 && maintenant - tBadge > DUREE_AFFICHAGE) ||
        (typeBadge == 2 && explosionFaite && maintenant - tBadge > DUREE_MOSFET)) {
      badgeActif = false;
      typeBadge = 0;
      FastLED.clear(); FastLED.show();
      digitalWrite(BROCHE_MOSFET, LOW); // Désactiver MOSFET
    }
    return; // on n'affiche pas le radar pendant l'effet
  }

  // === Radar escargot
  if (maintenant - dernier_tick >= DELAI_ESCARGOT) {
    dernier_tick = maintenant;
    radarLED();
  }

  // === NFC non bloquant (timeout court)
  uint8_t uid[7];
  uint8_t uidLength;
  bool success = nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength, 20); // 20 ms timeout

  if (success) {
    traiterBadge(uid, uidLength);
  }
}
